#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <setjmp.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <math.h>
#include "limits.h"

#define NULL 0
#define TRUE 1
#define BAD -1

#define NO_WHERE "/dev/null"
#define TERMINAL_OUT "/dev/tty"

#define TEMPLATE_SCREEN "screen"
#define MAX_TEMPLATE_LINE_LENGTH 1000
#define TEMPLATE_COMMAND_CHAR '*'
#define MAX_TEMPLATES 100
#define TEM_UNIT_COMMANDS "rteiu"
#define TEM_OTHER_COMMANDS "os"

#define MAX_STRING 80
#define MAX_EDITOR_LINES 100

#define MAX_ETA 100
#define MAX_DELAY 5
#define MAX_DWELL 2
#define MAX_OPEN_FILES 100

#define MAX_TOKENS 40
#define SEEK_CUR 1
#define DEFAULT_RMS 100
#define ACT_MIN -5
#define ACT_MAX 5

#define str_eq(x ,y) strcmp (x,y) == 0
#define str_not_eq(x, y) strcmp (x,y) != 0

/********************************** typdefs **********************************/

typedef char string[MAX_STRING];

typedef struct {		/* output template structure */
  string where;
  char line [MAX_TEMPLATE_LINE_LENGTH];
  FILE *fp;
  int modulus;
} template_type;


enum PT {INT, REAL, BOOLEAN};
typedef enum PT ParamType;	/* parameter list structure stuff */
struct Parameter_Struct {
  char *name;
  void *address;
  ParamType type;
  int min;
  int max;
};
typedef struct Parameter_Struct Parameter;

typedef void FunctionV ();	/* function list structure stuff */
typedef struct {
  char *name;
  FunctionV *func;
  char *flags;
} COMMAND;


/***********************************  FUNCTIONS  ***********************************/


int check_in_range (char type, int i);
int read_template (template_type templates[], char *fname, int num);
void read_template_line (char *to, char *line, char *where);
void close_templates (template_type templates[], int num_templates);
int get_val (char *str, int error);
void output_templates (template_type templates [], int num, double *ptr[], int pcp, int pos, int epochs);
int load_templates (template_type templates[], string tokens[MAX_TOKENS], int spot);
void add_unit_list (char *str, char *line, int max, int error, char *where);
void add_string (char *str, char *line, int max);

#ifdef READLINE
char *parameter_generator (char *text, int state);
char *command_generator (char* text, int state);
char *editor_generator (char *text, int state);
char *do_gets (char*);
void initialize_readline ();
void add_history (char*);
void editor (string tokens[MAX_TOKENS]);
#endif

				/* command line things */

void init_global (int argc, char *argv[], char *env[]);
void execute_command (string tokens[MAX_TOKENS]);
int search_tokens (string tokens[MAX_TOKENS], string command);
int get_str (string tokens[MAX_TOKENS], string opt_str);
char *bad_flag (string tokens[MAX_TOKENS], char *flag_list);
void err (int, char*);


				/* network things */

int get_epochs (string tokens[MAX_TOKENS]);
int get_update (string tokens[MAX_TOKENS]);
int get_random (string tokens[MAX_TOKENS]);
int get_test_flag (string tokens[MAX_TOKENS]);
int get_templates (string tokens[MAX_TOKENS], template_type templates[]);
int get_rms_flag (string tokens[MAX_TOKENS], FILE **rms_fp);
int get_lesion_list (string tokens[MAX_TOKENS], int lesion_list[]);
int get_stimulate_list (string tokens[MAX_TOKENS], int stimulate_list[]);
double get_margin (string tokens[MAX_TOKENS]);
void get_error (double teacher[], int TMask[]);
void c_teacher_forcing(double teacher[], int TMask[], int teacher_forcing_list []);
void c_update_stack_weights(double teacher[], int TMask[]);
void c_update_stack_activity(double current_input[], double noise);
void clamp_units (int list[], double val);

				/* stuff that shouldn't be here */
/*
int isdigit (char);
int isalpha (char);
int isatty (int);
int lrand48();
*/		
                               /* commands */

void display_source (string tokens[MAX_TOKENS]);
void display_patterns (string tokens[MAX_TOKENS]);
void load_network (string tokens[MAX_TOKENS]);
void display_parameters (string tokens[MAX_TOKENS]);
void graph_weights (string tokens[MAX_TOKENS]);
void initialize_weights (string tokens[MAX_TOKENS]);
void execute_script (string tokens[MAX_TOKENS]);
void quit (string tokens[MAX_TOKENS]);
void load_patterns (string tokens[MAX_TOKENS]);
void load_inputs (string tokens[MAX_TOKENS]);
void load_teachers (string tokens[MAX_TOKENS]);
void display (string tokens[MAX_TOKENS]);
void set (string tokens[MAX_TOKENS]);
void display_network (string tokens[MAX_TOKENS]);
void save_network (string tokens[MAX_TOKENS]);

void run_network (string tokens[MAX_TOKENS]);
void test_network (string tokens[MAX_TOKENS]);
void print (string tokens[MAX_TOKENS]);
void fix_weights (string tokens[MAX_TOKENS]);
void help (string tokens[MAX_TOKENS]);
void make_network_script (string tokens[MAX_TOKENS]);

                         /* text editor command */

int read_editor_file (char *fname, string lines[]);
int good_editor_command (char *com);


				/* misc */

void add_file (FILE *fp, char *fname);
double find_max_weights ();
void get_string (string tokens[MAX_TOKENS], int i, char *ret_str);
FILE *my_fopen (char *name, char *what);
int my_fclose (FILE *fp);
void close_all_files ();
char* get_parameter_string (int i);
int get_param_ptr (char *s_string);
void init_net_call ();
void exec_script (FILE *fp, string tokens[MAX_TOKENS]);
void open_init_file (string tokens[MAX_TOKENS]);
int check_state ();
char* convert_to_list (char *line);
int check_special (char *str);
void ctrl_c_handler ();
void continue_from_suspend ();
void bus_error_handler ();
int check_valid_filename (string fname);
double total_error ();
char peek_file (FILE *fp);
void skip_comments (FILE *fp);
int check_pattern_limit (int max, string fname);
int get_output_file (char *fname, int width);
int get_input_file (char *fname, int width);
double rnd ();
void turn_off_output ();
void turn_on_output ();
int fork();



				/* user defined pattern generators */

void pattern_generator_1 (double input [], double output [], int tmask [], int start);
void pattern_generator_2 (double input [], double output [], int tmask [], int start);
void pattern_generator_3 (double input [], double output [], int tmask [], int start);
void pattern_generator_4 (double input [], double output [], int tmask [], int start);

/******** network global variables **************/

double RW [MAX_COPIES] [MAX_UNITS] [MAX_UNITS];
double InW [MAX_COPIES] [MAX_UNITS] [MAX_INPUTS];
double InA [MAX_COPIES] [MAX_INPUTS];
double RA [MAX_COPIES] [MAX_UNITS];
double InWMask [MAX_UNITS] [MAX_INPUTS];
double RecWMask [MAX_UNITS] [MAX_UNITS];
double InWFix [MAX_UNITS] [MAX_INPUTS];
double RecWFix [MAX_UNITS] [MAX_UNITS];
double teacher [MAX_UNITS];
double error [MAX_UNITS];
double OldDelta [MAX_UNITS];
double NewDelta [MAX_UNITS];
double temp [MAX_UNITS];		
int cp;

/**************** PARAMETERS ********************/

int
  NET_UNITS,
  NET_COPIES,
  NET_INPUTS, 
  NET_OUTPUTS,
  DELAY,
  UPDATE, 
  TRAIN_ALL,
  PSOURCE,
  DWELL,
  RANDOM;

double
  USER0,
  USER1,
  USER2,
  USER3,
  USER4, 
  USER5,
  USER6,
  USER7,
  USER8,
  USER9,
  STIMULATE,
  LESION,
  BIAS,
  WEIGHT_RANGE,
  ETA;

/************** Other global variables ************/

string 
  FIX_LIST,
  INPUT_PATTERN_FILE,
  TEACHER_PATTERN_FILE,
  HOME_DIR,
  WEIGHT_FILE;


extern int FUNCTIONS;

int
  FILE_NO_LINES,
  FAKE_TMASK [MAX_UNITS],
  INIT_NET_FLAG,
  INIT_WEIGHT_FLAG,
  INIT_PATTERN_FLAG,
  AT_EDITOR,
  TMASKS [MAX_PATTERNS][MAX_UNITS],
  FILE_NUM,
  WAS_AT_PROMPT;  

double
  FAKE_PATTERN[MAX_UNITS],
  INPUTS[MAX_PATTERNS][MAX_INPUTS],
  TEACHERS[MAX_PATTERNS][MAX_UNITS];

extern Parameter 
  param_list [];

extern COMMAND
  commands[],
  editor_commands[];

extern FunctionV 
  *generators [];

FILE *OUT;

FILE *FILE_LIST[MAX_OPEN_FILES];

jmp_buf ENV, PROMPT_ENV;

