/* manexamp.c -- The examples which appear in the documentation are here. */

#include "bptt.h"
#include <readline/readline.h>
#include <readline/history.h>

/* A static variable for holding the line. */
static char *line_read = (char *)NULL;

/* Read a string, and return a pointer to it.  Returns NULL on EOF. */
char *do_gets (char *prompt)
{
  /* If the buffer has already been allocated, return the memory
     to the free pool. */
  if (line_read != (char *)NULL)
    {
      free (line_read);
      line_read = (char *)NULL;
    }
  /* Get a line from the user. */
  line_read = readline (prompt);

  /* If the line has any text in it, save it on the history. */
  if (line_read && *line_read)
    add_history (line_read);
  
  return (line_read);
}

/* **************************************************************** */
/*                                                                  */
/*                  Interface to Readline Completion                */
/*                                                                  */
/* **************************************************************** */

/* Tell the GNU Readline library how to complete.  We want to try to complete
   on command names if this is the first word in the line, or on filenames
   if not. */
void initialize_readline ()
{
  char **fileman_completion ();

  /* Allow conditional parsing of the ~/.inputrc file. */
  rl_readline_name = "FileMan";

  /* Tell the completer that we want a crack first. */
  rl_attempted_completion_function = (Function *)fileman_completion;
}

/* Attempt to complete on the contents of TEXT.  START and END show the
   region of TEXT that contains the word to complete.  We can use the
   entire line in case we want to do some simple parsing.  Return the
   array of matches, or NULL if there aren't any. */
char **
fileman_completion (text, start, end)
     char *text;
     int start, end;
{
  char **matches;
  char *command_generator ();
  char *parameter_generator ();

  matches = (char **)NULL;

  /* If this word is at the start of the line, then it is a command
     to complete.  Otherwise it is the name of a file in the current
     directory. */


  if (AT_EDITOR)
    matches = completion_matches (text, editor_generator);
  else
    if ((start == 0)||
	(strncmp (rl_line_buffer, "help ", 4) == 0))
      matches = completion_matches (text, command_generator);
    else
      if ((strncmp (rl_line_buffer, "set  ", 4) == 0) ||
	  (strncmp (rl_line_buffer, "print ", 6) == 0))
	matches = completion_matches (text, parameter_generator);

  return (matches);
}

/* Generator function for command completion.  STATE lets us know whether
   to start from scratch; without any state (i.e. STATE == 0), then we
   start at the top of the list. */
char *
command_generator (text, state)
     char *text;
     int state;
{
  static int list_index, len;
  char *name;

  /* If this is a new word to complete, initialize now.  This includes
     saving the length of TEXT for efficiency, and initializing the index
     variable to 0. */
  if (!state)
    {
      list_index = 0;      len = strlen (text);
    }

  /* Return the next name which partially matches from the command list. */
  while (name = commands[list_index].name)
    {
      list_index++;

      if (strncmp (name, text, len) == 0)
	return (name);
    }

  /* If no names matched, then return NULL. */
  return ((char *)NULL);
}

char *
editor_generator (text, state)
     char *text;
     int state;
{
  static int list_index, len;
  char *name;

  /* If this is a new word to complete, initialize now.  This includes
     saving the length of TEXT for efficiency, and initializing the index
     variable to 0. */
  if (!state)
    {
      list_index = 0;      len = strlen (text);
    }

  /* Return the next name which partially matches from the command list. */
  while (name = editor_commands[list_index].name)
    {
      list_index++;

      if (strncmp (name, text, len) == 0)
	return (name);
    }

  /* If no names matched, then return NULL. */
  return ((char *)NULL);
}


char *parameter_generator (text, state)
     char *text;
     int state;
{
  static int list_index, len;
  char *name;

  /* If this is a new word to complete, initialize now.  This includes
     saving the length of TEXT for efficiency, and initializing the index
     variable to 0. */
  if (!state)
    {
      list_index = 0;
      len = strlen (text);
    }

  /* Return the next name which partially matches from the command list. */
  while (name = param_list[list_index].name)
    {
      list_index++;

      if (strncmp (name, text, len) == 0)
	return (name);
    }

  /* If no names matched, then return NULL. */
  return ((char *)NULL);
}

/* Function which tells you that you can't do this. */
void too_dangerous (caller)
     char *caller;
{
  fprintf (stderr,
	   "%s: Too dangerous for me to distribute.  Write it yourself.\n",
	   caller);
}

/* Return non-zero if ARG is a valid argument for CALLER, else print
   an error message and return zero. */
int
valid_argument (caller, arg)
     char *caller, *arg;
{
  if (!arg || !*arg)
    {
      fprintf (stderr, "%s: Argument required.\n", caller);
      return (0);
    }

  return (1);
}


/*
 * Local variables:
 * compile-command: "cc -g -I../.. -L.. -o fileman fileman.c -lreadline -ltermcap"
 * end:
 */










