#include "bptt.h"


/* general error printing routine */
/* will display error message, close all open files */
/* and jump back to main menu */

void err (int i, char *str)
{
  fprintf (stderr, "\n   ERROR: ");
  switch (i)
    {
    case 0 :
      fprintf (stderr, "<%s> unrecognized command", str);
      break;
    case 1 :
      fprintf (stderr, "Invalid filename \"%s\"", str);
      break;
    case 2 :
      fprintf (stderr, "The -s flag must be specified");
      break;
    case 3 :
      fprintf (stderr, "Network Configuartion has not been specified");
      break;
    case 4 :
      fprintf (stderr, "Invalid parameter name");
      break;
    case 5 :
      fprintf (stderr, "Value out of range, parameter %s", str);
      break;
    case 6 :
      fprintf (stderr,  "Template file \"%s\" does not exist", str);
      break;
    case 7:
      fprintf (stderr,  "Template line or lines are invalid");
      break;
    case 8:
      fprintf (stderr,  "Weights have not been initialized");
      break;
    case 9:
      fprintf (stderr,  "Pattern source or files have not been specified");
      break;
    case 10:
      fprintf (stderr,  "Pattern file \"%s\" has too few lines or does not match", str);
      fprintf (stderr,  "\n          network configuration");
      break;
    case 11:
      fprintf (stderr, "Only bias has been specified as input (INPUTS = 1)");
      break;
    case 12:
      fprintf (stderr, "Invalid command flag <%s>", str);
      break;
    case 13:
      fprintf (stderr, "An error value must be specified after the -m flag");
      break;
    case 15:
      fprintf (stderr,  "Pattern sets have not been loaded");
      break;
    case 17:
      fprintf (stderr,  "Invalid output filename specified inside template");
      break;
    case 18:
      fprintf (stderr,  "No output units have been specified");
      break;
    case 19:
      fprintf (stderr, "Maximum number of templates exceeded (will continue)");
      break;
    case 20:
      fprintf (stderr, "Can not open \"%s\" for writing", str);
      break;
    case 22:
      fprintf (stderr, "Empty template file \"%s\"", str);
      break;
    case 23:
      fprintf (stderr, "Invalid command in unit list");
      break; /* disconnect units */
    case 24:
      fprintf (stderr, "Invalid editor command");
      break;
    case 25:
      fprintf (stderr, "Arguments out of range, please respecify");
      break; /* display patterns */
    case 26:
      fprintf (stderr, "With psource ON, you must specify the number of repeats");
      break; /* display patterns */
    case 27:
      fprintf (stderr, "DELAY must be evenly divisible by DWELL");
      break;			/* run net */
    case 28:
      fprintf (stderr, "Unit in lesion or stimulate list is out of range");
      break;			/* run net */
    case 29:
      fprintf (stderr, "At least one unit must be present in lesion or stimulate list");
      break;			/* run net */
    case 30:
      fprintf (stderr, "Too many patterns in \"%s\",  maximum size: %d", str, MAX_PATTERNS);
      break;
    case 31:
      fprintf (stderr, "Unit list or lists are out of range ", str);
      break;
    case 32:
      fprintf (stderr, "At least one template file must be specified with the -o option");
      break;
    case 33:
      fprintf (stderr, "Turn on /train-all/ or include a tmask specification in .teach file");
      break;
    case 34:
      fprintf (stderr, "Invalid filename specified with -B option");
      break;
    case 35:
      fprintf (stderr, "Filename must be specified with -B option");
      break;
    case 36:
      fprintf (stderr, "No functions have been compiled with BPTT");
      break;
    case 37:
      fprintf (stderr, "Number of patterns has not been specified in \"%s\"", str);
      break;
    case 38:
      fprintf (stderr, "Outputs cannot be larger then units, outputs set to units");
      break;
    case 39:
      fprintf (stderr, "Cannot find support file \"%s\"", str);
      break;
    }
  fprintf (stderr, "\n");

  close_all_files ();

  /* if piping input or output, terminate program */
  /* on error message */

  if (!(isatty(0)  && isatty (1)))
    {
      fprintf (stderr,"\nProgram Aborted\n\n");
      exit(1);
    }
  turn_on_output ();

  longjmp (ENV, 0);		/* return to main menu */
}





/* checks whether all flags in tokens[] are in flag_list */
/* returns NULL is flags are all good */
/* otherwise returns pointer to bad flag */

char *bad_flag (string tokens [], char *flag_list)
{
  char *flags[MAX_TOKENS];
  string temp;
  int i, j = 0, k, good;
  if (*flag_list)
    {
      strncpy (temp, flag_list, 80);
      flags [j] = strtok (temp, " ");
      while (flags[j])
	flags [++j] = strtok (NULL, " ");
      
      for (i=0; tokens[i][0]; i++)
	if (tokens[i][0] == '-')
	  {
	    good = NULL;
	    for (k=0;k<j;k++)
	      if (str_eq (flags[k],tokens[i]))
		good = TRUE;
	    if (!good) return tokens[i];
	  }
    }
  return NULL;
}


/* returns TRUE is string is a valid filename */
/* NULL otherwise */

int check_valid_filename (string fname)
{
  if (isdigit (fname[0]) ||
      (strpbrk (fname, " ")) ||
      (fname[0] == '-') ||
      (!fname[0]))
    return NULL;
  else
    return TRUE;
}


