#include "bptt.h"

/* get both pattern files */

void load_patterns (string tokens[])
{
  load_inputs (tokens);
  load_teachers (tokens);
}


/* main function to load input pattern file */

void load_inputs (string tokens [])
{
  int i;
  string fname;

  *INPUT_PATTERN_FILE = NULL;

  if (NET_INPUTS == 1)
    {
      fprintf (OUT,"\n   Warning: INPUTS = 1 so \".data\" file will not be loaded\n");
      strcpy (INPUT_PATTERN_FILE, "Not Needed");
      return;
    }

  FILE_NO_LINES = 0;

  sprintf (fname, "%s.data", tokens[1]);
  i = get_input_file (fname, NET_INPUTS);
  if (i) 
    {
      FILE_NO_LINES = i;
      sprintf (INPUT_PATTERN_FILE, "\"%s\"", fname);
    }
}



/* main function to load output pattern file */

void load_teachers (string tokens [])
{
  int i;
  string fname;

  *TEACHER_PATTERN_FILE = NULL;

  if (NET_OUTPUTS == 0)
    {
      fprintf (OUT,"\n   Warning: OUTPUTS = 0 so \".teach\" file will not be loaded\n");
      strcpy (TEACHER_PATTERN_FILE, "<Not Needed>");
      return;
    }

  sprintf (fname, "%s.teach", tokens[1]);

  FILE_NO_LINES = 0;

  i = get_output_file (fname, NET_OUTPUTS);

  if (i==BAD)
    err (10, fname);
  else
    if (i)
      {
	FILE_NO_LINES = i;
	sprintf (TEACHER_PATTERN_FILE, "\"%s\"", fname);
      }
}


/* reads in INPUTS array from fname */

int get_input_file (char *fname, int width)
{
  FILE *fp;
  int i, j, max = 0;

  if (!(fp = my_fopen (fname, "r")))
    err (1, fname);

  fscanf (fp, "%d", &max);

  max = check_pattern_limit (max, fname);

  if (max==0)
    err (37, fname);

  for (i=0; i<max; i++)
    {
      INPUTS[i][0] = BIAS;
      for (j=1; j<width; j++)
	if (!feof (fp))
	  fscanf (fp, "%lf", &INPUTS[i][j]);
	else
	  err (10, fname);
    }
  my_fclose (fp);
  return max;
}


/* reads in TEACHER and TMASK arrays from fname */

int get_output_file (char *fname, int width)
{
  FILE *fp;
  string temp;
  int i, j, max = 0;

  if (!(fp = my_fopen (fname, "r")))
    err (1, fname);
  
  fscanf (fp, "%d", &max);
  max = check_pattern_limit (max, fname);
  
  if (max==0)
    err (37, fname);

  for (i=0; i<max; i++)
    {
      for (j=0; j<width; j++)
	if (!feof (fp))
	  fscanf (fp, "%lf", &TEACHERS[i][j]);
	else
	  return BAD;

      if (!TRAIN_ALL)
	if (!feof (fp))
	  fscanf (fp, "%s", temp);
	else
	  return BAD;

      if (TRAIN_ALL || (str_eq (temp, "*")))
	for (j=0; j<width; j++)
	  TMASKS [i][j] = 1;
      else
	if (str_eq (temp, "|"))
	  {
	    for (j=0; j<width;j++)
	      if (!feof (fp))
		fscanf (fp, "%d", &TMASKS[i][j]);
	      else
		return BAD;
	  }
	else
	  if (str_eq (temp, "^"))
	    for (j=0; j<width; j++)
	      TMASKS[i][j] = 0;
	  else
	    err (33, "");
    }
  my_fclose (fp);
  return max;
}



/* save network architecutre and weights */

void save_network (string tokens[])
{
  int i,j,k, flag = TRUE;
  FILE *fp;

  if (tokens [1] == '\0') return;
  
  if (!strstr(tokens[1], ".wts")) strcat (tokens[1], ".wts");
  if (!check_valid_filename (tokens[1]))
    err (1, tokens[1]);

  if (str_eq (tokens [2], "-nc")) flag = NULL;

  fp = my_fopen (tokens[1], "w");

  if (flag) fprintf (fp, "(UNITS  COPIES  INPUTS   OUTPUTS)\n");
  fprintf (fp,"%d %d %d %d\n", NET_UNITS, NET_COPIES, NET_INPUTS, NET_OUTPUTS);
  fprintf (fp,"fix-weights %s\n", FIX_LIST);
  for (i=0; i<NET_COPIES; i++)
      for (j=0; j<NET_UNITS; j++)
	{
	  if (flag) fprintf (fp, "(C:%d R:%d from R%d-%d)\n", i, j, 0, NET_UNITS-1);
	  for (k=0; k<NET_UNITS; k++)
	    fprintf (fp, "%1.10f\n", RW[i][j][k]);
	      
	  if (flag) fprintf (fp, "(C:%d R:%d from I%d-%d)\n", i, j, 0, NET_INPUTS-1);
	  for (k=0; k<NET_INPUTS; k++)
	    fprintf (fp, "%1.10f\n", InW[i][j][k]);
	}
  my_fclose (fp);
  fprintf (OUT, "\n   Network weights saved to file <%s>\n", tokens[1]);
}


/* load network architecture and weights */


void load_network (string tokens[])
{
  int i,j,k;
  FILE *fp;
  string fix, fname;

  if (tokens [1] == '\0') return;
  
  if (!strstr(tokens[1], ".wts")) strcat (tokens[1], ".wts");
  fp = my_fopen (tokens[1], "r");
  if (!fp)
    err (1, tokens[1]);

  skip_comments (fp);
  fscanf (fp, "%d%d%d%d", &NET_UNITS, &NET_COPIES, &NET_INPUTS, &NET_OUTPUTS);
  init_net_call ();

				/* fix weights reading HACK! COUGH! */
  strcpy (fname, tokens[1]);
  turn_off_output ();
  fgets (fix, MAX_STRING, fp);
  fgets (fix, MAX_STRING, fp);
  fix[strlen(fix)-1] = 0;	/* get rid of \n */
  strcpy (FIX_LIST, fix+12);
  if (get_str (tokens, fix) == 1)
    *FIX_LIST = NULL;
  fix_weights (tokens);
  turn_on_output ();

  for (i=0; i<NET_COPIES; i++)
    for (j=0; j<NET_UNITS; j++)
      {
	skip_comments (fp);
	for (k=0; k<NET_UNITS; k++)
	  fscanf (fp, "%lf", &RW[i][j][k]);
	skip_comments (fp);
	for (k=0; k<NET_INPUTS; k++)
	  fscanf (fp, "%lf", &InW[i][j][k]);
      }
  my_fclose (fp);
  check_state();
  INIT_WEIGHT_FLAG = NULL;
  fprintf (OUT, "\n   Network weights loaded from file <%s>\n", fname);
  strncpy (WEIGHT_FILE, fname, MAX_STRING);
}
  

/* skips all lines beginning with `(` in a file */

void skip_comments (FILE *fp)
{
  string line;

  while ((peek_file (fp) == '(') ||
	 (peek_file (fp) == '\n'))
    fgets (line, MAX_STRING, fp);
}


int check_pattern_limit (int max, string fname)
{
  if (max<=MAX_PATTERNS)
    return max;
  else
    err (30, fname);
}



/* execute a script file */

void execute_script (string tokens [])
{
  int flag = NULL;
  FILE *fp;

  if (!(fp = my_fopen (tokens[1], "r")))  err (1, tokens[1]);

  if (!search_tokens (tokens, "-out"))
    {
      turn_off_output ();
      flag = TRUE;
    }

  exec_script (fp, tokens);

  my_fclose (fp);

  if (flag) turn_on_output ();
}


/* read lines from file and execute them */

void exec_script (FILE *fp, string tokens[])
{
  string temp;

  
  while (fgets (temp, MAX_STRING, fp))
    {
      check_state ();
      if (get_str (tokens, temp))
	execute_command (tokens);
    }
}

/* makes the network script file */

void make_network_script (string tokens[])
{
  FILE *fp;
  int i;

  if (tokens[1][0])
    {
      fp = my_fopen (tokens[1], "w");
      for (i=0;
	   param_list[i].name;
	   i++)
	{
	  fprintf (fp, "set %s ",
		   param_list[i].name);
	  switch (param_list[i].type)
	    {
	    case INT :
	      fprintf (fp, "%d", * (int*) param_list[i].address);
	      break;
	    case REAL :
	      fprintf (fp, "%6.3lf", * (double*) param_list[i].address);
	      break;
	    case BOOLEAN:
	      if (* (int*) param_list[i].address)
		fprintf (fp, "on");
	      else
		fprintf (fp, "off");
	    }
	  fprintf (fp, "\n");
	}
      fprintf (fp, "initialize-weights\n");
      if (*FIX_LIST)
	fprintf (fp, "fix-weights %s\n", FIX_LIST);
      my_fclose (fp);
    }
}
