#include "bptt.h"

FunctionV *generators[] = 
       {NULL, pattern_generator_1, pattern_generator_2,
	pattern_generator_3, pattern_generator_4};

/* list of all editor commands for completion */

COMMAND editor_commands[] = {
  {"add", NULL, ""},
  {"modify", NULL, ""},
  {"delete", NULL, ""},
  {"insert", NULL, ""},
  {"new", NULL, ""},
  {"list", NULL, ""},
  {"quit", NULL, ""},
  {"exit", NULL, ""},
  {NULL, NULL, ""}
};


/* list of all commands available for completion at main prompt*/
/* also contains a pointer to function linked to command name */
/* and a string indicating valid flags for each function */

COMMAND commands[] = {
  {"quit", quit, ""},
  {"display-patterns", display_patterns, ""},
  {"display-source", display_source, ""},
  {"load-patterns", load_patterns, ""},
  {"load-inputs", load_inputs, ""},
  {"load-teachers", load_teachers, ""},
  {"display-configuration", display, ""},
  {"initialize-weights", initialize_weights, "-s"},
  {"run-network", run_network, "-f -u -t -R -E -lesion -stimulate -o -s -M -B -n -nr"},
  {"test-network", test_network, "-f -u -t -R -E -lesion -stimulate -o -s -M -B -n -nr"},
  {"set", set, ""},
  {"display-network", display_network, ""},
  {"save-weights", save_network, "-nc"},
  {"load-weights", load_network, ""},
  {"execute-script", execute_script, "-out"},
  {"print", print, ""},
  {"display-parameters", display_parameters, ""},
  {"fix-weights", fix_weights, ""},
#ifdef READLINE
  {"editor", editor, ""},
#endif
  {"make-network-script", make_network_script, ""},
  {"graph-weights", graph_weights, ""},
  {(char*) NULL, (FunctionV*) NULL}
};


/* list of all parameter names */
/* contains pointer to variable address */
/* a type (REAL, INT, BOOLEAN) */
/* and a minimum and maximum value */
/* if min==max==NULL, any range is allowed */

Parameter param_list [] = {
  {"eta", &ETA, REAL, 0, MAX_ETA},
  {"bias", &BIAS, REAL, 0, 1},
  {"copies", &NET_COPIES, INT, 1, MAX_COPIES},
  {"units", &NET_UNITS, INT, 1, MAX_UNITS},
  {"inputs", &NET_INPUTS, INT, 1, MAX_INPUTS},
  {"outputs", &NET_OUTPUTS, INT, 0, MAX_UNITS},
  {"dwell", &DWELL, INT, 1, MAX_DWELL},
  {"delay", &DELAY, INT, 1, MAX_DELAY},
  {"update", &UPDATE, INT, NULL, NULL},
  {"psource", &PSOURCE, INT, 0, 4},
  {"lesion", &LESION, REAL, 0, 1},
  {"stimulate", &STIMULATE, REAL,0, 1},
  {"random", &RANDOM, BOOLEAN, NULL, NULL},
  {"train-all", &TRAIN_ALL, BOOLEAN, NULL, NULL},
  {"wrange", &WEIGHT_RANGE, REAL, 0, 10},
  {"user0", &USER0, REAL, NULL, NULL},
  {"user1", &USER1, REAL, NULL, NULL},
  {"user2", &USER2, REAL, NULL, NULL},
  {"user3", &USER3, REAL, NULL, NULL},
  {"user4", &USER4, REAL, NULL, NULL},
  {"user5", &USER5, REAL, NULL, NULL},
  {"user6", &USER6, REAL, NULL, NULL},
  {"user7", &USER7, REAL, NULL, NULL},
  {"user8", &USER8, REAL, NULL, NULL},
  {"user9", &USER9, REAL, NULL, NULL},
  {(char*) NULL, (void*) NULL, (int) NULL, NULL, NULL}
};



/* initializes all global variables and interrupts */

void init_global (int argc, char* argv[], char *env[])
{
  int i;


  /* get home directory from environment variable*/

  *HOME_DIR = NULL;
  for (i=0; env[i] != NULL; i++)
    if (strncmp (env[i], "HOME", 4) == 0)
      strncpy (HOME_DIR, env[i]+5, MAX_STRING);


  INIT_NET_FLAG = 1;
  INIT_WEIGHT_FLAG = 1;
  INIT_PATTERN_FLAG = 1;

  WEIGHT_RANGE = 1;
  USER0 = 0;
  USER1 = 0;
  USER2 = 0;
  USER3 = 0;
  USER4 = 0;
  USER5 = 0;
  USER6 = 0;
  USER7 = 0;
  USER8 = 0;
  USER9 = 0;
  STIMULATE = 1.0;
  LESION = 0.0;
  BIAS = 1.0;
  ETA = 1.0;
  NET_UNITS = 0;
  NET_COPIES = 0;
  NET_INPUTS = 0;
  NET_OUTPUTS = 0;
  DELAY = 1;
  UPDATE = 1000;
  TRAIN_ALL = TRUE;
  PSOURCE = 0;
  DWELL = 1;
  RANDOM = NULL;

  FILE_NUM = 0;
  WAS_AT_PROMPT = NULL;
  AT_EDITOR = NULL;
  OUT = stdout;

  *FIX_LIST = NULL,
  *INPUT_PATTERN_FILE = NULL,
  *TEACHER_PATTERN_FILE = NULL,
  *WEIGHT_FILE = NULL;

  /* these fake arrays are used when network is run */
  /* they exist because of DELAY, when a network is first */
  /* run, and DELAY > 1, there are some "empty" spots */
  /* that need to be filled, these arrays are those empty */
  /* spots */

  for (i=0; i<MAX_UNITS; i++)
    FAKE_PATTERN [i] = 0.5;
  for (i=0; i<MAX_UNITS; i++)
    FAKE_TMASK[i] = 0.0;

  /* initialize readline parameters */

#ifdef READLINE
  initialize_readline ();
#endif

  /* assign interrupt handlers */
  signal (SIGINT, ctrl_c_handler);
  signal (SIGCONT, continue_from_suspend);
  signal (SIGBUS, bus_error_handler);
}




/* reads the init file if one exists */

void open_init_file (string tokens[])
{
  FILE *fp;
  string temp;

  strncpy (temp, HOME_DIR, MAX_STRING);
  strcat (temp, "/.bpttinit");
  if (fp = my_fopen (temp, "r"))
    {
      fprintf (OUT, "Reading \".bpttinit\" file\n");
      setjmp (ENV);
      turn_off_output ();
      exec_script (fp, tokens);
      turn_on_output ();
      if (!my_fclose (fp))
	{
	  fprintf (stderr, "\n   Aborting...\n\n");
	  exit (1);
	}
    }
}

void quit (string tokens[])
{
  fprintf (OUT, "\n");
  exit (1);
}
