#include "bptt.h" 
  

/****************************************************************************/


void main (int argc, char *argv[], char *env[])
{
  string tokens[MAX_TOKENS];

  init_global (argc, argv, env);	  /* init global variables */

  fprintf (OUT,"\n\nWelcome to Back Propagation Through Time\nVersion 1.0\n\n");

  open_init_file (tokens);	  /* is there a .bpttinit file */

  for (;;)
    {
      setjmp (ENV);		  /* return here in case of error */
      check_state ();		  /* any state flags (NWP)? */
      if (get_str (tokens, (char*) NULL)) /* read line and break into TOKENS */
	                          /* if there is a line, execute tokens */
	execute_command (tokens); /* execute commands */
    }
}  



/****************************************************************************

  execute_command

  executes command line specified in tokens. Each spot in tokens contains
  a single word..i.e.

   bptt $ set units 20  = tokens[0] = "set"
                          tokens[1] = "units"
			  tokens[2] = "20"
			  tokens[3] = "\0"

*****************************************************************************/



void execute_command (string tokens[])
{
  int i;
  char *temp;
  FunctionV *function = NULL;

				/* search command list */
				/* and return ptr to func if find */
				/* else return NULL */

  for (i=0; commands[i].name; i++)
    if (str_eq (commands[i].name, tokens[0])) 
      {
	function = commands [i].func;
	break;
      }

  if (function)			
    if (temp = bad_flag (tokens, commands[i].flags))
      err (12, temp);		/* if user entered a bad flag, give an error */
    else
      (*function) (tokens);	/* call function */
  else
    if (tokens[0][0]) err (0, tokens[0]); /* else print error */
}


/****************************************************************************

  get_str

  reads a line from the keyboard or opt_str (if not NULL) and fills
  tokens with space separated words

  returns number of tokens read in

*****************************************************************************/


int get_str (string tokens[], string opt_str)
{
  int count = 0;
  char  *p;
  string
    command,
    pp,
    prompt,
    temp;

  for (count=0; count<MAX_TOKENS; count++)
    tokens[count][0] = NULL;
  
  count = 0;

				/* assemble prompt */

  if 
  (!PSOURCE)
    {
      strcpy (temp, "");
      if (*INPUT_PATTERN_FILE == NULL)
	strcpy (temp, "I");
      if (*TEACHER_PATTERN_FILE == NULL)
	strcpy (temp, "T");
      if ((*INPUT_PATTERN_FILE == NULL) && (*TEACHER_PATTERN_FILE == NULL))
	strcpy (temp, "P");
    }
  else
    {
      strcpy (temp, "");
    }

  sprintf (prompt, "\nbptt [%s%s%s%s%s]$ ",
	   (PSOURCE) ? "func": "file",
	   (INIT_NET_FLAG || INIT_WEIGHT_FLAG || INIT_PATTERN_FLAG) ? "-" : "",
	   (INIT_NET_FLAG == 0) ? "" : "N",
	   (INIT_WEIGHT_FLAG == 0) ? "" : "W",
	   temp);

  /* if opt_str is NULL, get line from stdin */
  if (!opt_str)
    {

      /* is NULL is read in and we are reading from the keyboard */
      /* return NULL */
      /* else if NULL is read in and we are reading from a file */
      /* that means EOF was reached, terminate program */
      /* if characters are read in and we are reading from a file */
      /* echo the characters */
      WAS_AT_PROMPT = TRUE;

#ifdef READLINE
      if (!(p = do_gets (prompt)))
	if (isatty (0))		
	  return NULL;
	else
	  {			
	    fprintf (OUT, "\n\n"); 
	    exit (1);
	  }
#else
      printf ("%s", prompt);
      if (gets (pp))
	p = pp;
      else
	p = NULL;

      if (!p)
	{			
	  fprintf (OUT, "\n\n"); 
	  exit (1);
	}      
#endif
      else
	if (!isatty(0))
	  fprintf (OUT, "%s\n", p);
      WAS_AT_PROMPT = NULL;
    }
  else
    {
      /* we are executing a script file, print prompt and script command */
      fprintf (OUT,"%s%s", prompt, opt_str);
      p = opt_str;
    }

  /* break command line into words and store each word in a position */
  /* in the array TOKENS */

  strncpy (command, p, MAX_STRING);
  if (p = strtok (command, "\n "))
    {
      strncpy (tokens[count++], p, MAX_STRING);
      while (p = strtok (NULL, "\n "))
	strncpy (tokens[count++], p, MAX_STRING);
    }
  
  return count;
}



/* checks if flags (NWP) need to be changed, 
keeps static copies of parameters to decide if any
 values have changes */

int check_state ()
{
  static int net_units = 0,
             net_outputs = 0,
             net_inputs = 0,
             net_layers = 0;
  int pattern,
      weight,
      net;

  pattern = INIT_PATTERN_FLAG;
  weight = INIT_WEIGHT_FLAG;
  net = INIT_NET_FLAG;

  if ((NET_UNITS != net_units) ||
      (NET_OUTPUTS != net_outputs) ||
      (NET_INPUTS != net_inputs) ||
      (NET_COPIES != net_layers))
    INIT_WEIGHT_FLAG = 1;

  if ((NET_UNITS == 0) ||
      (NET_COPIES == 0) ||
      (NET_OUTPUTS == 0))
    INIT_NET_FLAG = 1;
  else
    INIT_NET_FLAG = 0;

  if (PSOURCE || (NET_INPUTS == 1)) 
    INIT_PATTERN_FLAG = 0;
  else
    if (FILE_NO_LINES)
      INIT_PATTERN_FLAG = 0;
    else
      INIT_PATTERN_FLAG = 1;

  if ((net_inputs != NET_INPUTS) ||
      (net_outputs != NET_OUTPUTS))
    INIT_PATTERN_FLAG = 1;
      
  net_units = NET_UNITS;
  net_layers = NET_COPIES;
  net_inputs = NET_INPUTS;
  net_outputs = NET_OUTPUTS;

  if (PSOURCE && (FUNCTIONS != 1))
    {
      PSOURCE = 0;
      err (36, "");
    }

  if (NET_OUTPUTS > NET_UNITS)
    {
      NET_OUTPUTS = NET_UNITS;
      err (38, "");
    }

  if ((pattern != INIT_PATTERN_FLAG) ||
      (weight != INIT_WEIGHT_FLAG) ||
      (net != INIT_NET_FLAG))
    return TRUE;
  else
    return NULL;
}

/****************************************************************************

  search_tokens

  will search tokens for specified command and return integer index of
  next position

  i.e.  tokens[0] = "run-network"
        tokens[1] = "-s"
	tokens[2] = "20"
	tokens[3] = "\0"

  search_token (tokens, "-s") would return 2

  function returns a zero if the flag is not found

*****************************************************************************/


int search_tokens (string tokens[], string command)
{
  int p;
  for (p=0; *tokens [p] != '\0'; p++)
    if (strcmp (tokens[p], command) == 0) return (p+1);
  return NULL;
}

