#include "bptt.h"
#define GAUSS_NUM 5   

double gauss (double range)
{
  int i;
  double num;

  num = 0;

  for (i=0; i<GAUSS_NUM; i++)
    num = num + (2 * (rnd () - 0.5));

  return (num / GAUSS_NUM) * range;
}


void c_update_stack_activity(double current_input[], double noise)
{
  int NextCp, PreviousCp;
  register int i,j;
  double net;
  double exp();

  for (j = 0; j < NET_INPUTS; j++)
    InA[cp][j] = *current_input++;

  if (cp + 1 < NET_COPIES) NextCp = cp + 1;
  else NextCp = 0;
  if (cp > 0) PreviousCp = cp - 1;
  else PreviousCp = NET_COPIES - 1;

  for (i = 0; i < NET_UNITS; i++) {
    net = 0;
    for (j = 0; j < NET_INPUTS; j++)
      net = net + (InW[cp][i][j] * InA[cp][j]);
    for (j = 0; j < NET_UNITS; j++)
      net = net + (RW[cp][i][j] * RA[PreviousCp][j]);
    
    if (noise) net = net + gauss (noise);
    
    if (net > ACT_MAX) net = ACT_MAX;
    else if (net < ACT_MIN) net = ACT_MIN;

    RA[cp][i] = (double) 1.0 / (1.0 + exp((double) - net));

  }

  for (i = 0; i < NET_UNITS; i++) {
    for (j = 0; j < NET_INPUTS; j++)
      InW[NextCp][i][j] = InW[cp][i][j];
    for (j = 0; j < NET_UNITS; j++)
      RW[NextCp][i][j] = RW[cp][i][j];
  }
  cp = NextCp;
}

void c_update_stack_weights(double teacher[], int TMask[])
{
  int PreviousCp, FoundA1, h, a, post, pre, Cminus1;
  register int i,j;
  double FPrime, s, etad;

  Cminus1 = NET_COPIES - 1;


  if (cp > 0) PreviousCp = cp - 1;
  else PreviousCp = Cminus1;

  FoundA1 = NULL;
  for (i = 0; i < NET_UNITS && !FoundA1; i++)
    FoundA1 = TMask[i] == 1;

  if (FoundA1) {
    for (i = 0; i < NET_UNITS; i++) {
      error[i] = TMask[i] * (teacher[i] - RA[PreviousCp][i]);
      OldDelta[i] = error[i] * (1.0 - RA[PreviousCp][i]) * RA[PreviousCp][i];
    }
        
    if (PreviousCp > 0) a = PreviousCp - 1;
    else a = Cminus1;

    for (i = 0; i < NET_UNITS; i++) {
      for (j = 0; j < NET_INPUTS; j++)
	InW[cp][i][j] = InW[cp][i][j] + (ETA * OldDelta[i] * InA[PreviousCp][j]);
      for (j = 0; j < NET_UNITS; j++)
	RW[cp][i][j] = RW[cp][i][j] + (ETA * OldDelta[i] * RA[a][j]);
    }

    post = cp;
    pre = PreviousCp;
    for (h = 1; h < (Cminus1); h++) {
      if (post > 0) post = post - 1;
      else post = Cminus1;
      if (pre > 0) pre = pre - 1;
      else pre = Cminus1;

      for (j = 0; j < NET_UNITS; j++) {
	FPrime = RA[pre][j] * (1.0 - RA[pre][j]);
	s = 0.0;
	for (i = 0; i < NET_UNITS; i++)
	  s = s + (OldDelta[i] * RW[post][i][j]);
	NewDelta[j] = s * FPrime;
       }
      if (pre > 0) a = pre - 1;
      else a = Cminus1;
      for (i = 0; i < NET_UNITS; i++) {
	etad = ETA * NewDelta[i];
	for (j = 0; j < NET_INPUTS; j++)
	  InW[cp][i][j] = InW[cp][i][j] + (etad * InA[pre][j]);
	for (j = 0; j < NET_UNITS; j++)
	  RW[cp][i][j] = RW[cp][i][j] + (etad * RA[a][j]);
      }
      for (i = 0; i < NET_UNITS; i++)
	{
	  temp[i] = OldDelta[i];
	  OldDelta[i] = NewDelta[i];
	  NewDelta[i] = temp[i];
	}
    }
				/* fix weights */
				/* if neccessary */

    if (*FIX_LIST)
      for (h=0; h<NET_COPIES; h++)
	for (i=0; i<NET_UNITS; i++)
	  {
	    for (j=0; j<NET_UNITS; j++)
	      if (RecWMask[i][j])
		RW [h][i][j] = RecWFix[i][j];
	    for (j=0; j<NET_INPUTS; j++)
	      if (InWMask[i][j])
		InW [h][i][j] = InWFix[i][j];
	  }
  }
}

void c_teacher_forcing(double teacher[], int TMask[], int teacher_forcing_list[])
{
  int ForceCp, i;

  if (cp > 0) ForceCp = cp - 1;	
  else ForceCp = NET_COPIES - 1;

  for (i = 0; i < NET_UNITS; i++)
    if (teacher_forcing_list[i] == 1) RA[ForceCp][i] = teacher[i];
}

void get_error (double teacher[], int TMask[])
{
  int i, pcp;

  if (cp>0)  pcp = cp-1; else pcp = NET_COPIES-1;
  
  for (i=0; i<NET_UNITS; i++)
    error [i] = TMask[i]*(RA[pcp][i]-teacher[i]);
}


