#include "bptt.h"

void run_network (string tokens [])
{

  int sweeps = 0,		/* sweep counter */
      max_sweeps = 0,		/* number specified by -s flag */
      update = 0,
      random = 0,
      pcp = 0,			/* points to previous copy */
      done = 0,
      pos = 0,			/* holds current position in delay array */
				/* pos - 1 (cyclic array) contains current */
				/* pattern to train on, pos contains newest patern */
      test_flag = 0,
      i = 0,
      m = 0,
      pid = 0,
      num_templates = 0,
      teacher_forcing_flag = 0,
      pattern_pos = 0,
      rms_flag = 0, 
      dwell_counter = 1,
      delay_mod = 0,
      user_tmask [MAX_DELAY][MAX_UNITS],
      lesion_list [MAX_UNITS + 1], /* list of units to lesion */
      stimulate_list [MAX_UNITS + 1], /* list of units to stimulate */
      teacher_forcing_list [MAX_UNITS + 1],
      lesion_flag = NULL,
      stimulate_flag = NULL,
      *ptr_tmask [MAX_DELAY],
       fname_index = 0;

  template_type templates[MAX_TEMPLATES];

  double *ptr [MAX_DELAY],	/* array of pointers to teacher arrays */
				/* used because of delay */
         *current_input,
         user_input [MAX_INPUTS], /* interface arrays for internal pattern functions */
         user_output [MAX_DELAY][MAX_UNITS], /* more of the same */
         rms_error = 0,
         avg_rms = 0,
         noise = 0,
         margin = 0;
         

  FILE *rms_fp;

				/* check for following error conditions */

  if (INIT_NET_FLAG) err (3, ""); /* no network */
  if (INIT_WEIGHT_FLAG) err (8, ""); /* no weights */
  if (INIT_PATTERN_FLAG) err (9, ""); /* no patterns */
  if (DELAY % DWELL) err (27, ""); /* delay & dwell not correct */
 
				/* fill up array pointing to inputs and teachers */
				/* with pointers to "blank" arrays */

  for (i=0; i < DELAY; i++)
    {
      ptr_tmask [i] = FAKE_TMASK;
      ptr [i] = FAKE_PATTERN;
    }

  delay_mod = DELAY / DWELL;

  if (i = search_tokens (tokens, "-B"))
    {
      if (!check_valid_filename (tokens[i]))
	err (34, "");
      fprintf (OUT,"\n   BPTT backgrounded...\n");
      pid = fork ();
      if (pid)
	{
	  fprintf (OUT,"\n   Process ID: [%d]\n\n", pid);
	  exit (1);
	}
      else
	{
	  turn_off_output ();
	  fname_index = i;
	}
    }

  if (i = search_tokens (tokens, "-n"))
    {
      noise = atof (tokens[i]);
      srand48 (time(NULL));
    }
  
				/* check the if the following flags are present */
				/* flag check functions can be found in run_network_flags.c */

  max_sweeps = get_epochs (tokens);
  margin = get_margin (tokens);
  update = get_update (tokens);
  random = get_random (tokens);
  test_flag = get_test_flag (tokens);
  num_templates = get_templates (tokens, templates);
  rms_flag = get_rms_flag (tokens, &rms_fp);
  lesion_flag = get_lesion_list (tokens, lesion_list);
  stimulate_flag = get_stimulate_list (tokens, stimulate_list);

  if (i = search_tokens (tokens, "-f"))
    {
      teacher_forcing_flag = 1;
      for (m=0; m<NET_UNITS; m++)
	if (m<NET_OUTPUTS)
	  teacher_forcing_list [m] = 1;
	else
	  teacher_forcing_list [m] = 0;
      while (isdigit (tokens[i][0]))
	{
	  m = atoi (tokens[i++]);
	  if ((m>=NET_OUTPUTS) || (m<0))
	    err (25, "");
	  else
	    teacher_forcing_list [m] = 0;
	}
    }
  else
    teacher_forcing_flag = 0;


				/* if margin supplied, but not -E, use default of 100 */

  if ((margin>0) && (!rms_flag))
    {
      rms_flag = 100;
      rms_fp = OUT;
    }

  if (!(search_tokens (tokens, "-nr")))
    {
      for (i=0; i < NET_COPIES; i++)
	for (m=0; m < NET_UNITS; m++)
	  RA[i][m] = 0.0;
    }

				/* if functions are the pattern source, initialize interface*/
				/* arrays */

  if (PSOURCE)
    {
      user_input [0] = BIAS;
      for (i=1; i< NET_INPUTS; i++)
	user_input [i] = 0.0;
      for (m=0; m<MAX_DELAY; m++)
	for (i=0; i< NET_OUTPUTS; i++)
	  user_output [m][i] = 0.0;
      for (m=0; m<MAX_DELAY; m++)
	for (i=0; i<NET_UNITS; i++)
	  if (i<NET_OUTPUTS) 
	    user_tmask [m][i] = 1;
	  else 
	    user_tmask[m][i] = 0;
      (*generators[PSOURCE]) (user_input, user_output [pos], user_tmask [pos], TRUE);
    }

  fprintf (OUT,"\n");

  max_sweeps ++;

  for (sweeps = 1;
       (sweeps < max_sweeps) && (!done);
       sweeps++)
    {
				/* if it is time for a new input pattern, lets get one */
      if (--dwell_counter == 0)
	{
	  dwell_counter = DWELL;

				/* check is source is functions or files */
	  if (PSOURCE) 
	    {
				/* if functions, call the appropriate internal function */
				/* interface arrays are passed in */

	      (*generators [PSOURCE]) (user_input, user_output [pos], user_tmask [pos], NULL);
	      current_input = user_input;
	      ptr [pos] = user_output [pos];
	      ptr_tmask [pos] = user_tmask [pos];
	    }
	  else
	    {

				/* if files, grab the right pattern */

	      current_input = INPUTS [pattern_pos];
	      ptr [pos] = TEACHERS [pattern_pos];
	      ptr_tmask [pos] = TMASKS [pattern_pos];
	      if (random)
		pattern_pos = lrand48 () % FILE_NO_LINES;
	      else
		if (++pattern_pos == FILE_NO_LINES)
		  pattern_pos = 0;
	    }
	  pos = (pos+1) % delay_mod;
	}

				/* create activation */
      c_update_stack_activity (current_input, noise);

      if (lesion_flag) clamp_units (lesion_list, LESION);
      if (stimulate_flag) clamp_units (stimulate_list, STIMULATE);

				/* is it time to train? */
      if (dwell_counter == 1)
	if (!test_flag)
	  {
				/* train the network */
	    c_update_stack_weights (ptr[pos], ptr_tmask [pos]);
	    if (rms_flag) rms_error = rms_error + total_error ();
	  }
	else
	  {
	    get_error (ptr[pos], ptr_tmask[pos]);
	    if (rms_flag) rms_error = rms_error + total_error ();
	  }

				/* get the current activation copy # */
      pcp = (cp > 0) ? cp-1 : NET_COPIES-1;

				/* check if it is time to average the error */
      if (rms_flag && !(sweeps % rms_flag))
	{
	  avg_rms = rms_error / rms_flag * DWELL;
	  fprintf (rms_fp, "   %8.5lf\n", avg_rms);
	  fflush (rms_fp);
	  if (margin && (avg_rms < margin)) done = TRUE;
	  rms_error = 0;
	}

				/* produce output if templates are being used */
      if (num_templates) output_templates (templates, num_templates, ptr, pcp, pos, sweeps); 

				/* teacher force if neccessary */
      if ((teacher_forcing_flag)  && (dwell_counter == 1)) c_teacher_forcing (ptr[pos], ptr_tmask[pos], teacher_forcing_list); 

      if ((update>0) && ((sweeps % update) == 0))
	fprintf (OUT, "   Sweeps Completed: %d\n", sweeps);

    } /*  main sweep loop */

				/* all done with sweeps */
				/* if templates  */
				/* or error files are being used */
				/* we need to close all the files */

  if (rms_flag && (rms_fp != stdout))
    my_fclose (rms_fp);

  if (num_templates)
    close_templates (templates, num_templates);

				/* was the -B (background) flag used */
  if (fname_index)
    {
				/* create a token array  suitable for save-weights command */
      strncpy (tokens[1], tokens[fname_index], MAX_STRING);
      save_network (tokens);
      turn_on_output ();
      fprintf (OUT, "\n\n   BPTT exiting...\n\n");
      exit (1);
    }

  if (!margin)
    fprintf (OUT,"\n   * Completed sweeps *\n");
  else
    fprintf (OUT,"\n   * Margin %s met in %d sweeps *\n", 
	    (done) ? "" : "*NOT*",
	    sweeps-1);
}



void clamp_units (int list[], double val)
{
  int i, j;
  
  for (j=0; j<NET_COPIES; j++)
    for (i=0; list[i] != BAD; i++)
      RA[j][ list[i] ] = val;
}



void test_network (string tokens [])
{
  int i;

  for (i=0; (i<MAX_TOKENS) && tokens[i][0]; i++);
  strcpy (tokens[i], "-t");
  run_network (tokens);
}
