#include "bptt.h"

int get_val (char *str, int error)
{
  if (!(*str)) return 0;
  switch (*str)
    {
    case 'I' :
      return NET_INPUTS-1;
    case 'O' :
    case 'T' :
      return NET_OUTPUTS-1;
    case 'U' :
      return NET_UNITS-1;
    default  :
      if (!isdigit (*str))
	err (error, "");
      else
	return atoi (str);
    }
}
      
int check_in_range (char type, int i)
{
  int err_flag = NULL;

  switch (type)
  {
    case 'i' : if ((i<0) || (i>=NET_INPUTS)) err_flag = TRUE;
      break;
    case 'r' : if ((i<0) || (i>=NET_UNITS)) err_flag = TRUE;
      break;
    case 't' : if ((i<0) || (i>=NET_OUTPUTS)) err_flag = TRUE;
  }

  if (err_flag)
    err (31, "");

  return !err_flag;
}
 
void add_string (char *str, char *line, int max)
{
  if ((strlen (str) + strlen (line)) > max)
    return;
  else
    strcat (str, line);
}

void add_unit_list (char *str, char *line, int max, int error, char *where)
{
  int i, start, end;
  char type;
  string temp;

  type = *line++;
  if (strchr (TEM_OTHER_COMMANDS, type))
    {
      sprintf (temp, "%c%c%s", TEMPLATE_COMMAND_CHAR, type, line);
      add_string (str, temp, max);
      return;
    }
  if (!strchr (TEM_UNIT_COMMANDS, type))
    err (7, "");
  start = get_val (line, error);
  line = strpbrk (line, "-");
  if (line)
    if (!(++line))
      err (7, "");
    else
      end = get_val (line, error);
  else
    end = start;

  for (i=start; i<=end; i++)
    if (!check_in_range (type, i))
      err (31, where);
    else
      {
	sprintf (temp, "%c%c%d  ", TEMPLATE_COMMAND_CHAR, type, i);
	add_string (str, temp, max);
      }
}
	  
    

void add_char (char *str, char add, int max)
{
  int i;
  i = strlen (str);
  if (i == (max-1)) return;
  str [i] = add;
  str [i+1] = NULL;
}

void read_template_line (char *to, char *line, char *where)
{
  string temp;
  char *r;

  if (strpbrk (line, "\""))
    {
      if (!check_valid_filename (r = strtok (line, "\"")))
	err (17, "");
      strncpy (where, r, MAX_STRING);
      line = strtok (NULL, "\"");
    }
  else
    strcpy (where, TEMPLATE_SCREEN);

  *to = NULL;

  while (*line)
    if (*line == '(')
      {
	*temp = NULL;
	while (*(++line) != ')')
	  if (*line)
	    add_char (temp, *line, MAX_STRING);
	  else
	    err (7, "");
	line ++;
	add_unit_list (to, temp, MAX_TEMPLATE_LINE_LENGTH, 7, "in template file");
      }
    else
      add_char (to, *line++, MAX_TEMPLATE_LINE_LENGTH);
}



int read_template (template_type templates[], char *fname, int num)
{
  FILE *fp;
  char line [MAX_TEMPLATE_LINE_LENGTH];
  char temp;

  if (!(fp = my_fopen (fname, "r"))) err (6, fname);
  
  while (fgets (line, MAX_TEMPLATE_LINE_LENGTH, fp))
    {
      if (strlen (line) > 1)
	{
	  read_template_line (templates[num].line, line, templates[num].where);
	  num++;
	}
      if (!feof (fp))
	{
	  fseek (fp, -1, SEEK_CUR);
	  while ((temp = fgetc (fp)) != EOF)
	    if (temp == '\n')
	      break;
	}
    }
  my_fclose (fp);
  if (!num)  err (22, fname);
  return num;
}

int load_templates (template_type templates[], string tokens[], int spot)
{
  int num = 0, num2 = 0, i;

  while (*tokens[spot] && (*tokens [spot] != '-'))
    {
      num2 = read_template (templates, tokens [spot++], num);
      if ((isdigit (tokens [spot][0])) && atoi (tokens[spot]))
	{
	  for (i=num; i < num2; i++)
	    templates [i].modulus = atoi (tokens[spot]);
	  spot++;
	}
      else
	for (i=num; i < num2; i++)
	  templates [i].modulus = 1;
      num = num2;
    }

  if (num == 0) err (32, "");

  for (i=0; i<num; i++)
    if (str_eq (templates[i].where, TEMPLATE_SCREEN))
      templates[i].fp = OUT;
    else
      templates[i].fp = my_fopen (templates[i].where, "w");
  return num;
}

void close_templates (template_type templates[], int num_templates)
{
  int i;

  for (i=0; i < num_templates; i++)
    if (str_not_eq (templates[i].where, TEMPLATE_SCREEN))
      my_fclose (templates[i].fp);
}


double total_error ()
{
  int i;
  double sum = 0;

  for (i=0; i<NET_OUTPUTS; i++)
    sum = sum + (error [i] * error [i]);
  return sqrt (sum/NET_OUTPUTS);
}


void output_templates (template_type templates [], int num, double *ptr[], int pcp, int pos, int epochs)
{

  FILE *out;
  char *output_str;
  int i;
  double scalar, offset;

  for (i=0; i < num; i++)
    if (!(epochs % templates[i].modulus))
      {
	scalar = 1;
	offset = 0;
	out = templates [i].fp;
	output_str = templates[i]. line;
	while (*output_str != NULL)
	  if (! (*output_str == TEMPLATE_COMMAND_CHAR))
	    fprintf (out, "%c", *output_str++);
	  else
	    {
	      switch (*++output_str)
		{
		case 'r':
		  fprintf (out,"%6.3lf", offset + scalar*RA[pcp][atoi (++output_str)]);
		  break;
		case 'i':
		  fprintf (out,"%6.3lf", offset + scalar*InA[pcp][atoi (++output_str)]);
		  break;
		case 't': 
		  fprintf (out,"%6.3lf", offset + scalar*ptr[pos][atoi (++output_str)]);
		  break;
		case 'e':
		  fprintf (out, "%6.3lf", offset + scalar*error[atoi (++output_str)] * error [atoi (output_str)]);
		  break;
		case 'E': 
		  fprintf (out, "%6.3lf", offset + scalar*total_error ());
		  ++output_str;
		  break;
		case 's' :
		  scalar = (atof (++output_str));
		  break;
		case 'o' :
		  offset = (atof (++output_str));
		  break;
		}
	      while (isdigit (*output_str) ||
		     (strpbrk (output_str, ".-")))
		output_str++;
	    }
      }
}

